import sys

from cqu_kb.config.config import config, Config
from cqu_kb.utils import check_user, log, check_output_path, select_core
from cqu_kb.version import __version__


def main(username, password, path):
    core = select_core(username)
    cal = core(username, password).main()

    with open(path, "wb") as f:
        f.write(cal.to_ical())

    log(f"课表已经保存到{path}")


def server_main(username, password, path):
    main(username, password, path)


def local_main():
    username, password = check_user()
    check_output_path()
    main(username, password, config["output"]["path"])


def console_main():
    import argparse

    def parse_args() -> argparse.Namespace:
        """Parse the command line arguments for the `cqu jwc` binary.

        :return: Namespace with parsed arguments.
        """
        parser = argparse.ArgumentParser(
            prog="kb",
            description="第三方 重庆大学 课表导出工具",
        )

        parser.add_argument(
            "-v",
            "--version",
            action="version",
            version=f"CQU_kb {__version__}",
            help="显示版本号",
        )
        parser.add_argument(
            "-c",
            "--config_path",
            help="查询配置文件路径",
            action="store_true",
        )
        parser.add_argument(
            "-r",
            "--reset",
            help="重置配置项",
            action="store_true",
        )
        parser.add_argument(
            "-u",
            "--username",
            help="学号",
            type=int,
            default=config["user_info"]["username"],
        )
        parser.add_argument(
            "-p",
            "--password",
            help="密码",
            type=str,
            default=config["user_info"]["password"],
        )
        parser.add_argument(
            "-o",
            "--output",
            help="课表输出路径",
            type=str,
            default=config["output"]["path"],
        )
        return parser.parse_args()

    args = parse_args()
    if args.reset:
        Config.reset()
        log("已重置配置文件")
    if args.config_path:
        log(f"配置文件位于 {Config.path}\n")
        sys.exit()

    config.dump()

    local_main()


if __name__ == "__main__":
    local_main()



from urllib import parse

from bs4 import BeautifulSoup as BS
from cqu_jxgl import Student

from cqu_kb.core.KBCore import KBCore


class KBCoreUnderGrad(KBCore):
    def _get_payload(self, student):
        """
        This function prepares the payload to be sent in HTTP POST method. There are three main fields in the payload. The
        only thing that should be paid attention to is "Sel_XNXQ" field, which denotes the years（学年） and terms（学期）.

        :param student: an instance of Student from cqu-jxgl
        :return: payload to be used in HTTP POST
        """
        # First Get. Won't get any information but a cookie.
        response = student.get("/znpk/Pri_StuSel.aspx")
        soup = BS(response.content, features="html.parser")
        return parse.urlencode(
            {
                "Sel_XNXQ": soup.find("option")["value"],
                "rad": "on",
                "px": "1",
            }
        )

    def main(self):
        student = Student(username=self.username, password=self.password)

        student.login()
        page_content = student.post(
            url="/znpk/Pri_StuSel_rpt.aspx",
            headers={"Content-Type": "application/x-www-form-urlencoded;charset=utf-8"},
            data=self._get_payload(student),
        ).content
        course_info = self.get_course_info(page_content)
        return self.get_cal(course_info)

    def get_course_info(self, page_content):
        soup = BS(page_content.decode("gbk"), features="html.parser")
        tables = soup.find_all(class_="page_table")

        courses_info = []
        for i in range(len(tables) // 2):
            table = tables[i * 2 + 1]
            rows = table.find("tbody").find_all("tr")
            cols_num = len(rows[0].find_all("td"))
            for row in rows:
                course_info = {}
                cols = row.find_all("td")
                course_info.update(
                    {
                        "course_name": cols[1].text.split("]")[-1]
                        + ("" if cols_num == 13 else "（实验）")
                    }
                )
                course_info.update({"teacher": cols[-4 if cols_num == 13 else -5].text})
                course_info.update({"weeks": cols[-3].text})
                course_info.update({"time": cols[-2].text.replace("节]", "")})
                course_info.update({"location": cols[-1].text})
                courses_info.append(course_info)
        for i in range(len(courses_info)):
            if (
                courses_info[i]["course_name"] == ""
                or courses_info[i]["course_name"] == "（实验）"
            ):
                courses_info[i]["course_name"] = courses_info[i - 1]["course_name"]
        return courses_info



import re

from bs4 import BeautifulSoup as BS
from requests import Session

from cqu_kb.core.KBCore import KBCore


class KBCoreGrad(KBCore):
    def _get_payload(self):
        return {
            "userId": self.username,
            "password": self.password,
            "userType": "student",
            "imageField.x": "50",
            "imageField.y": "6",
        }

    def main(self):
        # 实例化 requests.Session 对象
        s = Session()

        # url 配置
        base_url = "http://mis.cqu.edu.cn/mis"
        login_url = base_url + "/login.jsp"
        select_course_url = base_url + "/student/plan/select_course.jsp"
        cal_base_url = base_url + "/curricula/show_stu.jsp?stuSerial="

        # 系统登录
        s.post(login_url, self._get_payload())

        # 获取 stuSerial 参数
        res = s.get(select_course_url)
        stu_serial = re.findall(r"stuSerial=(\d*)?", res.text)[0]

        # 获取课表 html
        cal_url = cal_base_url + stu_serial
        cal = s.get(cal_url)

        course_info = self.get_course_info(cal.text)
        return self.get_cal(course_info)

    def get_course_info(self, page_content):
        page_content = page_content.replace("<br/>", "[br]").replace("<br>", "[br]")

        soup = BS(page_content, features="html.parser")
        table = soup.find_all("table")[0]

        courses_info = []
        for line in table.find_all("tr")[2:]:
            for i, element in enumerate(line.find_all("td")[1:-1]):
                courses = tuple(filter(lambda x: x != "", element.text.split("[br]")))
                for j in range(len(courses) // 7):
                    courses_info.append(
                        {
                            "course_name": courses[j * 7 + 2][3:],
                            "teacher": courses[j * 7 + 5][3:],
                            "weeks": courses[j * 7 + 3][3:-1].replace(" ", ","),
                            "time": "{}[{}".format(
                                i, courses[j * 7 + 4][3:]
                            ),  # compromise with UnderGrad core
                            "location": courses[j * 7 + 6][3:],
                        }
                    )
        return courses_info



import uuid
from abc import abstractmethod, ABC
from datetime import datetime, timedelta

import pytz
from icalendar import Calendar, Event
from icalendar import Timezone, vDDDTypes

from cqu_kb.config import config


class KBCore(ABC):
    def __init__(self, username, password):
        self.password = password
        self.username = username

    @abstractmethod
    def _get_payload(self, **kwargs):
        pass

    @abstractmethod
    def main(self):
        pass

    @abstractmethod
    def get_course_info(self, page_content):
        pass

    @staticmethod
    def _add_datetime(component, name, time):
        vdatetime = vDDDTypes(time)
        if "VALUE" in vdatetime.params and "TZID" in vdatetime.params:
            vdatetime.params.pop("VALUE")
        component.add(name, vdatetime)

    def get_cal(self, course_info):
        chinese_to_numbers = {"一": 0, "二": 1, "三": 2, "四": 3, "五": 4, "六": 5, "日": 6}

        coursenum_to_time = {
            "1": (timedelta(hours=8, minutes=30), timedelta(hours=9, minutes=15)),
            "2": (timedelta(hours=9, minutes=25), timedelta(hours=10, minutes=10)),
            "3": (timedelta(hours=10, minutes=30), timedelta(hours=11, minutes=15)),
            "4": (timedelta(hours=11, minutes=25), timedelta(hours=12, minutes=10)),
            "5": (timedelta(hours=13, minutes=30), timedelta(hours=14, minutes=15)),
            "6": (timedelta(hours=14, minutes=25), timedelta(hours=15, minutes=10)),
            "7": (timedelta(hours=15, minutes=20), timedelta(hours=16, minutes=5)),
            "8": (timedelta(hours=16, minutes=25), timedelta(hours=17, minutes=10)),
            "9": (timedelta(hours=17, minutes=20), timedelta(hours=18, minutes=5)),
            "10": (timedelta(hours=19, minutes=0), timedelta(hours=19, minutes=45)),
            "11": (timedelta(hours=19, minutes=55), timedelta(hours=20, minutes=40)),
            "12": (timedelta(hours=20, minutes=50), timedelta(hours=21, minutes=35)),
            "13": (timedelta(hours=21, minutes=45), timedelta(hours=22, minutes=30)),
        }

        term_start_time = datetime(
            year=int(config["term_start_time"]["year"]),
            month=int(config["term_start_time"]["month"]),
            day=int(config["term_start_time"]["day"]),
            tzinfo=pytz.timezone("Asia/Shanghai"),
        )

        events = []

        for course in course_info:
            week_nums = []
            week_segs = course["weeks"].split(",")
            for seg in week_segs:
                delimiter = [int(i) for i in seg.split("-")]
                start = delimiter[0]
                end = delimiter[1] if len(delimiter) == 2 else start
                for i in range(start, end + 1):
                    week_nums.append(i)
            if not course["time"].split("[")[0].isdigit():
                day = chinese_to_numbers[course["time"].split("[")[0]]
            else:
                day = int(course["time"].split("[")[0])
            seg = course["time"].split("[")[1].split("-")
            if len(seg) == 2:
                inweek_delta_start = timedelta(days=day) + coursenum_to_time[seg[0]][0]
                inweek_delta_end = timedelta(days=day) + coursenum_to_time[seg[1]][1]
                for week_num in week_nums:
                    event_start_datetime = (
                        term_start_time
                        + (week_num - 1) * timedelta(days=7)
                        + inweek_delta_start
                    )
                    event_end_datetime = (
                        term_start_time
                        + (week_num - 1) * timedelta(days=7)
                        + inweek_delta_end
                    )
                    event = Event()
                    event.add(
                        "summary",
                        "[{}]{}".format(course["teacher"], course["course_name"]),
                    )
                    event.add("location", course["location"])
                    self._add_datetime(event, "dtstart", event_start_datetime)
                    self._add_datetime(event, "dtend", event_end_datetime)
                    # Fix #2: 添加 dtstamp 与 uid 属性
                    event.add("dtstamp", datetime.utcnow())
                    namespace = uuid.UUID(
                        bytes=int(event_start_datetime.timestamp()).to_bytes(
                            length=8, byteorder="big"
                        )
                        + int(event_end_datetime.timestamp()).to_bytes(
                            length=8, byteorder="big"
                        )
                    )
                    event.add(
                        "uid",
                        uuid.uuid3(
                            namespace, f"{course['course_name']}-{course['teacher']}"
                        ),
                    )

                    events.append(event)

        cal = Calendar()
        cal.add(
            "prodid",
            f'-//重庆大学课表//{config["user_info"]["username"]}//Powered By cqu-kb//',
        )
        cal.add("version", "2.0")
        cal.add_component(
            Timezone.from_ical(
                "BEGIN:VTIMEZONE\n"
                "TZID:Asia/Shanghai\n"
                "X-LIC-LOCATION:Asia/Shanghai\n"
                "BEGIN:STANDARD\n"
                "TZNAME:CST\n"
                "DTSTART:16010101T000000\n"
                "TZOFFSETFROM:+0800\n"
                "TZOFFSETTO:+0800\n"
                "END:STANDARD\n"
                "END:VTIMEZONE\n"
            )
        )
        for event in events:
            cal.add_component(event)
        return cal
